from abc import ABC
from abc import ABCMeta
from abc import abstractmethod
from dataclasses import dataclass
from typing import Union

from django.contrib import admin
from django.contrib.admin.options import ModelAdmin
from django.db import models
from unfold.admin import ModelAdmin as UnfoldModelAdmin
from unfold.contrib.forms.widgets import WysiwygWidget


@dataclass
class FieldPermissions:
    visible: Union[bool, tuple] = False
    editable: Union[bool, tuple] = False


class DynamicAdminFields(ABC):
    @abstractmethod
    def get_field_rules(self, request, obj=None) -> dict[str, FieldPermissions]:
        """Must be implemented by child classes to define field rules"""

    def get_fieldsets(self, request, obj=None):
        field_rules = self.get_field_rules(request, obj)
        base_fieldsets = super().get_fieldsets(request, obj)

        if not base_fieldsets:
            all_fields = self.get_fields(request, obj)
            visible_fields = [
                field for field in all_fields
                if field_rules.get(field, FieldPermissions()).visible
            ]
            return ((None, {"fields": visible_fields}),) if visible_fields else ()

        filtered_fieldsets = []
        for name, options in base_fieldsets:
            if not isinstance(options, dict) or "fields" not in options:
                continue

            visible_fields = [
                field for field in options["fields"]
                if field_rules.get(field, FieldPermissions()).visible
            ]

            if visible_fields:
                new_options = options.copy()
                new_options["fields"] = visible_fields
                filtered_fieldsets.append((name, new_options))

        return tuple(filtered_fieldsets)

    def get_readonly_fields(self, request, obj=None):
        field_rules = self.get_field_rules(request, obj)
        base_readonly = super().get_readonly_fields(request, obj)

        readonly_fields = list(base_readonly)
        for field, permissions in field_rules.items():
            if permissions.visible and not permissions.editable:
                readonly_fields.append(field)

        return tuple(readonly_fields)

    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj, **kwargs)
        field_rules = self.get_field_rules(request, obj)

        for field in list(form.base_fields):
            if not field_rules.get(field, FieldPermissions()).visible:
                del form.base_fields[field]

        return form

    def get_list_display(self, request):
        field_rules = self.get_field_rules(request)
        base_list_display = getattr(self, "list_display", ())
        return tuple(
            field for field in base_list_display
            if field not in field_rules or field_rules[field].visible
        )

    def get_list_editable(self, request):
        """
        Filters the ModelAdmin's list_editable fields using the field rules.
        """
        field_rules = self.get_field_rules(request)
        # Retrieve the current list_editable (if defined)
        list_editable = getattr(self, "list_editable", ())
        # Only include fields that are marked editable in our field rules.
        return tuple(
            field for field in list_editable
            if field_rules.get(field, FieldPermissions()).editable
        )

    def changelist_view(self, request, extra_context=None):
        """
        Override the changelist view to dynamically update list_editable.
        """
        if hasattr(self, "list_editable"):
            self.list_editable = self.get_list_editable(request)
        return super().changelist_view(request, extra_context=extra_context)



class BaseModelAdminMeta(ModelAdmin.__class__, ABCMeta):
    pass


class BaseModelAdmin(
    DynamicAdminFields,
    UnfoldModelAdmin,
    ABC,
    metaclass=BaseModelAdminMeta,
):
    def __init__(self, model, admin_site):
        super().__init__(model, admin_site)
        self.filter_horizontal = [field.name for field in model._meta.many_to_many]

    empty_value_display = "-"
    show_facets = admin.ShowFacets.ALWAYS
    formfield_overrides = {
        models.TextField: {
            "widget": WysiwygWidget,
        },
    }
    compressed_fields = True
    warn_unsaved_form = True

    @abstractmethod
    def has_add_permission(self, request):
        pass

    @abstractmethod
    def has_change_permission(self, request, obj=None):
        pass

    @abstractmethod
    def has_delete_permission(self, request, obj=None):
        pass



# serializer_generator.py
from django.apps import apps

from django_model_suite.generators.base import BaseGenerator


class SerializerGenerator(BaseGenerator):
    def __init__(self, app_name: str, model_name: str, base_path: str):
        super().__init__(app_name, model_name, base_path)
        self.model = apps.get_model(app_label=self.app_name, model_name=self.model_name)

    def generate(self, fields: list) -> None:
        model_import_path = f"{self.model.__module__}"
        content = f"""from rest_framework import serializers
from {model_import_path} import {self.model_name_capital}

class {self.model_name_capital}MinimalSerializer(serializers.ModelSerializer):
    class Meta:
        model = {self.model_name_capital}
        fields = (id, )
        read_only = True

class {self.model_name_capital}DetailedSerializer(serializers.ModelSerializer):
    class Meta:
        model = {self.model_name_capital}
        fields = (id,)
        read_only = True

class {self.model_name_capital}CreateSerializer(serializers.ModelSerializer):
    class Meta:
        model = {self.model_name_capital}
        fields = ()

class {self.model_name_capital}UpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = {self.model_name_capital}
        fields = ()
"""
        self.write_file("serializers.py", content)


from ..base import BaseGenerator


class ChangeViewGenerator(BaseGenerator):
    def generate(self, fields: list) -> None:
        content = f'''class {self.model_name_capital}ChangeView:
    filter_horizontal = ()
    compressed_fields = True
    autocomplete_fields = ()
    fieldsets = (
        ("Information", {{"fields": ()}}),
    )
'''
        self.write_file('change_view.py', content)


from typing import List
from django.apps import apps
from ..base import BaseGenerator


class PermissionsGenerator(BaseGenerator):
    def __init__(self, app_name: str, model_name: str, base_path: str):
        super().__init__(app_name, model_name, base_path)
        self.model = apps.get_model(app_label=self.app_name, model_name=self.model_name)

    def generate(self, fields: List[str]) -> None:
        field_rules = [
            f"            {self.model_name_capital}Fields.{field.upper()}: FieldPermissions(\n"
            "                visible=(\n"
            "                    context.is_superuser\n"
            "                ),\n"
            "                editable=(\n"
            "                ),\n"
            "            )" for field in fields
        ]

        content = f"""from typing import Optional, Dict
from django.http import HttpRequest
from ...fields.{self.model_name_lower} import {self.model_name_capital}Fields
from ....core.admin import FieldPermissions
from {self.model.__module__} import {self.model_name_capital}
from .context import {self.model_name_capital}ContextLogic


class {self.model_name_capital}Permissions:
    def has_add_permission(self, request: HttpRequest) -> bool:
        return False

    def has_change_permission(self, request: HttpRequest, {self.model_name_lower}: Optional[{self.model_name_capital}] = None) -> bool:
        return False

    def has_delete_permission(self, request: HttpRequest, {self.model_name_lower}: Optional[{self.model_name_capital}] = None) -> bool:
        return False

    def get_field_rules(self, request: HttpRequest, {self.model_name_lower}: Optional[{self.model_name_capital}] = None) -> Dict:
        context = {self.model_name_capital}ContextLogic(request, {self.model_name_lower})

        return {{
""" + ',\n'.join(field_rules) + """
        }
"""
        self.write_file('permissions.py', content)


from django_model_suite.generators.base import BaseGenerator


class PaginationGenerator(BaseGenerator):
    def generate(self, fields: list) -> None:
        content = f"""from rest_framework.pagination import CursorPagination

class {self.model_name_capital}Pagination(CursorPagination):
    page_size = 10
    ordering = '-id'
    page_size_query_param = 'page_size'
    max_page_size = 100
"""
        self.write_file("pagination.py", content)



# service_generator.py
from django.apps import apps

from ..base import BaseGenerator


class ServiceGenerator(BaseGenerator):
    def __init__(self, app_name: str, model_name: str, base_path: str):
        super().__init__(app_name, model_name, base_path)
        self.output_path = "domain/services"
        self.model = apps.get_model(app_label=self.app_name, model_name=self.model_name)

    def generate(self, fields: list) -> None:
        model_import_path = f"{self.model.__module__}"
        content = f'''from typing import Any
from {model_import_path} import {self.model_name_capital}

class {self.model_name_capital}Service:
    @staticmethod
    def create_{self.model_name_lower}(*, data: dict[str, Any]) -> {self.model_name_capital}:
        validated_data = validate_{self.model_name_lower}_create(data=data)
        instance = {self.model_name_capital}.objects.create(**validated_data)
        return instance

    @staticmethod
    def update_{self.model_name_lower}(*, instance: {self.model_name_capital}, data: dict[str, Any]) -> {self.model_name_capital}:
        validated_data = validate_{self.model_name_lower}_update(instance=instance, data=data)
        for attr, value in validated_data.items():
            setattr(instance, attr, value)
        instance.save()
        return instance

    @staticmethod
    def delete_{self.model_name_lower}(*, instance: {self.model_name_capital}) -> None:
        instance.delete()
'''
        self.write_file(f'{self.model_name_lower}.py', content)


from django.apps import apps
from django_model_suite.generators.base import BaseGenerator


class ViewGenerator(BaseGenerator):
    def __init__(self, app_name: str, model_name: str, base_path: str):
        super().__init__(app_name, model_name, base_path)
        self.model = apps.get_model(app_label=self.app_name, model_name=self.model_name)

    def generate(self, fields: list) -> None:
        model_import_path = f"{self.model.__module__}"
        content = f'''from rest_framework import generics, status
from rest_framework.response import Response
from {model_import_path} import {self.model_name_capital}
from ...domain.services.{self.model_name_lower} import {self.model_name_capital}Service
from ...domain.selectors.{self.model_name_lower} import {self.model_name_capital}Selector
from .serializers import (
    {self.model_name_capital}MinimalSerializer,
    {self.model_name_capital}DetailedSerializer,
    {self.model_name_capital}CreateSerializer,
    {self.model_name_capital}UpdateSerializer
)
from .filters import {self.model_name_capital}Filter
from .pagination import {self.model_name_capital}Pagination

class {self.model_name_capital}ListView(generics.ListAPIView):
    serializer_class = {self.model_name_capital}MinimalSerializer
    filterset_class = {self.model_name_capital}Filter
    pagination_class = {self.model_name_capital}Pagination

    def get_queryset(self):
        return {self.model_name_capital}Selector.get_queryset()

class {self.model_name_capital}CreateView(generics.CreateAPIView):
    serializer_class = {self.model_name_capital}CreateSerializer

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        instance = {self.model_name_capital}Service.create_{self.model_name_lower}(
            data=serializer.validated_data
        )
        response_serializer = {self.model_name_capital}DetailedSerializer(instance)
        return Response(response_serializer.data, status=status.HTTP_201_CREATED)

class {self.model_name_capital}DetailView(generics.RetrieveAPIView):
    serializer_class = {self.model_name_capital}DetailedSerializer
    lookup_field = 'id'

    def get_object(self):
        return {self.model_name_capital}Selector.by_id(id=self.kwargs['id'])

class {self.model_name_capital}UpdateView(generics.UpdateAPIView):
    serializer_class = {self.model_name_capital}UpdateSerializer
    lookup_field = 'id'

    def get_object(self):
        return {self.model_name_capital}Selector.by_id(id=self.kwargs['id'])

    def update(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=kwargs.get('partial', False))
        serializer.is_valid(raise_exception=True)
        updated_instance = {self.model_name_capital}Service.update_{self.model_name_lower}(
            instance=instance,
            data=serializer.validated_data
        )
        response_serializer = {self.model_name_capital}DetailedSerializer(updated_instance)
        return Response(response_serializer.data)

class {self.model_name_capital}DeleteView(generics.DestroyAPIView):
    lookup_field = 'id'

    def get_object(self):
        return {self.model_name_capital}Selector.by_id(id=self.kwargs['id'])

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        {self.model_name_capital}Service.delete_{self.model_name_lower}(instance=instance)
        return Response(status=status.HTTP_204_NO_CONTENT)
'''
        self.write_file(f'views.py', content)


import importlib
import importlib.util
import os
import sys
from pathlib import Path

from django.apps import AppConfig
from django.apps import apps


class CoreConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'appss.core'

    def ready(self):
        # Loop over all installed apps
        for app_config in apps.get_app_configs():
            admin_dir = Path(app_config.path) / "admin"
            if admin_dir.is_dir():
                admin_pkg = f"{app_config.name}.admin"
                # If the admin package isn’t already in sys.modules,
                # create a dummy package so that the dotted module names can be resolved.
                if admin_pkg not in sys.modules:
                    spec = importlib.util.spec_from_loader(admin_pkg, loader=None, origin=str(admin_dir))
                    module = importlib.util.module_from_spec(spec)
                    # Make sure Python treats this as a package
                    module.__path__ = [str(admin_dir)]
                    sys.modules[admin_pkg] = module

                # Import all .py files in the admin directory (except __init__.py)
                for file in admin_dir.glob("*.py"):
                    if file.name != "__init__.py":
                        module_name = f"{admin_pkg}.{file.stem}"
                        importlib.import_module(module_name)

                # Now import .py files from subdirectories of admin
                for subdir in admin_dir.iterdir():
                    if subdir.is_dir():
                        sub_pkg = f"{admin_pkg}.{subdir.name}"
                        # If the subpackage isn’t in sys.modules (because, say, __init__.py is missing),
                        # register it as a namespace package.
                        if sub_pkg not in sys.modules:
                            spec = importlib.util.spec_from_loader(sub_pkg, loader=None, origin=str(subdir))
                            module = importlib.util.module_from_spec(spec)
                            module.__path__ = [str(subdir)]
                            sys.modules[sub_pkg] = module

                        for file in subdir.glob("*.py"):
                            if file.name != "__init__.py":
                                module_name = f"{sub_pkg}.{file.stem}"
                                importlib.import_module(module_name)

        # Import models in a similar way.
        for app_config in apps.get_app_configs():
            self.import_modules_from_directory(
                app_config.path,
                "models",
                app_config.name,
            )

    def import_modules_from_directory(self, base_path, subdirectory, app_name):
        directory_path = Path(base_path) / subdirectory
        if directory_path.exists():
            modules = [
                f[:-3]  # strip .py
                for f in os.listdir(directory_path)
                if f.endswith(".py") and not f.startswith("__")
            ]
            for module in modules:
                importlib.import_module(f"{app_name}.{subdirectory}.{module}")



from django.apps import apps

from ..base import BaseGenerator


class ValidatorGenerator(BaseGenerator):
    def __init__(self, app_name: str, model_name: str, base_path: str):
        super().__init__(app_name, model_name, base_path)
        self.output_path = "domain/validators"
        self.model = apps.get_model(app_label=self.app_name, model_name=self.model_name)

    def generate(self, fields: list) -> None:
        model_import_path = f"{self.model.__module__}"
        content = f'''from typing import Any
from django.core.exceptions import ValidationError
from {model_import_path} import {self.model_name_capital}

def validate_{self.model_name_lower}_create(*, data: dict[str, Any]) -> dict[str, Any]:
    # Add your validation logic here
    return data

def validate_{self.model_name_lower}_update(*, instance: {self.model_name_capital}, data: dict[str, Any]) -> dict[str, Any]:
    # Add your validation logic here
    return data
'''
        self.write_file(f'{self.model_name_lower}.py', content)


import os

from django.apps import apps
from django.core.management import BaseCommand

from django_model_suite.generators.admin.admin_generator import AdminGenerator
from django_model_suite.generators.admin.change_view_generator import ChangeViewGenerator
from django_model_suite.generators.admin.context_generator import ContextGenerator
from django_model_suite.generators.admin.display_generator import DisplayGenerator
from django_model_suite.generators.field.fields_generator import FieldsGenerator
from django_model_suite.generators.admin.list_view_generator import ListViewGenerator
from django_model_suite.generators.admin.permissions_generator import PermissionsGenerator
from django_model_suite.generators.api.filter_generator import FilterGenerator
from django_model_suite.generators.api.pagination_generator import PaginationGenerator
from django_model_suite.generators.api.serializer_generator import SerializerGenerator
from django_model_suite.generators.api.url_generator import URLGenerator
from django_model_suite.generators.api.view_generator import ViewGenerator
from django_model_suite.generators.domain.selector_generator import SelectorGenerator
from django_model_suite.generators.domain.service_generator import ServiceGenerator
from django_model_suite.generators.domain.validator_generator import ValidatorGenerator
from django_model_suite.generators.model_utils import get_model_fields


class Command(BaseCommand):
    COMPONENT_CONFIGS = {
        "fields": {
            "path_template": "fields/",
            "generators": [
                FieldsGenerator,
            ],
        },
        "admin": {
            "path_template": "admin/{model}/",
            "generators": [
                ListViewGenerator,
                ChangeViewGenerator,
                PermissionsGenerator,
                ContextGenerator,
                DisplayGenerator,
                AdminGenerator,
            ],
        },
        "api": {
            "path_template": "api/{model}/",
            "generators": [
                SerializerGenerator,
                ViewGenerator,
                URLGenerator,
                FilterGenerator,
                PaginationGenerator,
            ],
        },

        "selectors": {
            "path_template": "domain/selectors/",
            "generators": [SelectorGenerator],
        },
        "services": {
            "path_template": "domain/services/",
            "generators": [ServiceGenerator],
        },
        "validators": {
            "path_template": "domain/validators/",
            "generators": [ValidatorGenerator],
        },
    }

    def add_arguments(self, parser):
        parser.add_argument("app_name", type=str, help="Name of the app (e.g., user)")
        parser.add_argument("model_name", type=str, help="Name of the model")

    def get_app_path(self, app_name: str) -> str:
        try:
            app_config = apps.get_app_config(app_name)
            app_path = os.path.dirname(app_config.module.__file__)
            self.stdout.write(f"Found app path: {app_path}")
            return app_path
        except LookupError:
            raise ValueError(f"App '{app_name}' not found in INSTALLED_APPS")

    def handle(self, *args, **options):
        app_name = options["app_name"]
        model_name = options["model_name"]

        try:
            app_path = self.get_app_path(app_name)
            if not app_path:
                raise ValueError("App path cannot be empty")

            fields = get_model_fields(app_name, model_name)

            for component, config in self.COMPONENT_CONFIGS.items():
                self._generate_component(
                    app_path, app_name, model_name, component, config, fields
                )

            self.stdout.write(
                self.style.SUCCESS(f"Successfully generated files for model '{model_name}'")
            )
        except Exception as e:
            self.stdout.write(self.style.ERROR(f"Error: {e}"))

    def _generate_component(
            self,
            app_path: str,
            app_name: str,
            model_name: str,
            component: str,
            config: dict,
            fields: list,
    ) -> None:
        path_template = config["path_template"].format(model=model_name.lower())
        base_path = os.path.join(app_path, path_template)
        self.stdout.write(f"Generating {component} in {base_path}")

        generators = [
            generator_class(app_name, model_name, base_path)
            for generator_class in config["generators"]
        ]

        for generator in generators:
            generator.generate(fields)



from typing import List

from django.apps import apps

from ..base import BaseGenerator


class ContextGenerator(BaseGenerator):
    def __init__(self, app_name: str, model_name: str, base_path: str):
        super().__init__(app_name, model_name, base_path)
        self.model = apps.get_model(app_label=self.app_name, model_name=self.model_name)

    def generate(self, fields: List[str] = None) -> None:  # Add fields parameter with default None
        content = f"""from typing import Optional
from django.http import HttpRequest
from {self.model.__module__} import {self.model_name_capital}


class {self.model_name_capital}ContextLogic:
    def __init__(self, request: HttpRequest, {self.model_name_lower}: Optional[{self.model_name_capital}] = None):
        self.request = request
        self.{self.model_name_lower} = {self.model_name_lower}

    @property
    def is_superuser(self) -> bool:
        return self.request.user.is_superuser

    @property
    def is_staff(self) -> bool:
        return self.request.user.is_staff

    @property
    def is_creating(self) -> bool:
        return self.{self.model_name_lower} is None
"""
        self.write_file('context.py', content)



from django.apps import apps

from ..base import BaseGenerator


class SelectorGenerator(BaseGenerator):
    def __init__(self, app_name: str, model_name: str, base_path: str):
        super().__init__(app_name, model_name, base_path)
        self.output_path = "domain/selectors"
        self.model = apps.get_model(app_label=self.app_name, model_name=self.model_name)

    def generate(self, fields: list) -> None:
        model_import_path = f"{self.model.__module__}"
        content = f'''from django.db.models import QuerySet
from {model_import_path} import {self.model_name_capital}

class {self.model_name_capital}Selector:
    @staticmethod
    def get_queryset() -> QuerySet:
        return {self.model_name_capital}.objects.all()

    @staticmethod
    def by_id(*, id: int) -> {self.model_name_capital}:
        return {self.model_name_capital}Selector.get_queryset().get(id=id)
'''
        self.write_file(f'{self.model_name_lower}.py', content)


import os

from django.apps import apps


def get_model_fields(app_name: str, model_name: str) -> list:
    """Get non-relational fields from a model."""
    model = apps.get_model(app_name, model_name)
    return [field.name for field in model._meta.get_fields() if not field.is_relation]


def ensure_package(path: str) -> None:
    os.makedirs(path, exist_ok=True)
    init_file = os.path.join(path, "__init__.py")
    if not os.path.exists(init_file):
        with open(init_file, "w"):
            pass



from django_model_suite.generators.base import BaseGenerator


class FieldsGenerator(BaseGenerator):
    def generate(self, fields: list) -> None:
        field_lines = [f"    {field.upper()} = \"{field}\"" for field in fields]
        content = [
            f"class {self.model_name_capital}Fields:",
            *field_lines,
            "",
            "    @classmethod",
            "    def get_field_name(cls, model, field):",
            "        return model._meta.get_field(field).name"
        ]
        self.write_file(f'{self.model_name_lower}.py', '\n'.join(content))



from django.apps import apps

from ..base import BaseGenerator


class DisplayGenerator(BaseGenerator):
    def __init__(self, app_name: str, model_name: str, base_path: str):
        super().__init__(app_name, model_name, base_path)
        self.model = apps.get_model(app_label=self.app_name, model_name=self.model_name)

    def generate(self, fields: list) -> None:
        model_import_path = f"{self.model.__module__}"
        content = f'''from unfold.decorators import display
from {model_import_path} import {self.model_name_capital}

class {self.model_name_capital}DisplayMixin:
    @display(description="{self.model_name_lower}", header=True)
    def display_header(self, {self.model_name_lower}: {self.model_name_capital}):
        """Display header with image if available."""
        return [
            {self.model_name_lower}.pk,
            "",
            "O",
            {{"path": {self.model_name_lower}.image.url if hasattr({self.model_name_lower}, 'image') and {self.model_name_lower}.image else None}},
        ]
'''
        self.write_file('display.py', content)


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="django_model_suite",
    version="0.1.3",
    author="Omar Gawdat",
    author_email="omargawdaat@gmail.com",
    description="A Django app for generating boilerplate code",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/omargawdat/django-blueprint",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    python_requires=">=3.8",
    install_requires=[
        "Django>=3.2",
        "django-unfold>=0.5.0",
        "djangorestframework>=3.14.0",
        "django-filter>=23.0",
    ],
)


# admin_generator.py
from django.apps import apps

from ..base import BaseGenerator


class AdminGenerator(BaseGenerator):
    def __init__(self, app_name: str, model_name: str, base_path: str):
        super().__init__(app_name, model_name, base_path)
        self.model = apps.get_model(app_label=self.app_name, model_name=self.model_name)

    def generate(self, fields: list) -> None:
        model_import_path = f"{self.model.__module__}"
        content = f'''from django.contrib import admin
from unfold.admin import ModelAdmin
from .list_view import {self.model_name_capital}ListView
from .change_view import {self.model_name_capital}ChangeView
from .permissions import {self.model_name_capital}Permissions
from .display import {self.model_name_capital}DisplayMixin
from {model_import_path} import {self.model_name_capital}
from ....core.admin import BaseModelAdmin

@admin.register({self.model_name_capital})
class {self.model_name_capital}Admin(
    {self.model_name_capital}DisplayMixin,
    {self.model_name_capital}ListView,
    {self.model_name_capital}ChangeView,
    {self.model_name_capital}Permissions,
    BaseModelAdmin,
):
    pass
'''
        self.write_file('admin.py', content)


from django_model_suite.generators.base import BaseGenerator


class FilterGenerator(BaseGenerator):
    def generate(self, fields: list) -> None:
        content = f"""from django_filters import rest_framework as filters

class {self.model_name_capital}Filter(filters.FilterSet):
    pass
"""
        file_name = "filters.py"
        self.write_file(file_name, content)



import os
from abc import ABC, abstractmethod

from django_model_suite.generators.model_utils import ensure_package


class BaseGenerator(ABC):
    def __init__(self, app_name: str, model_name: str, base_path: str):
        self.app_name = app_name
        self.model_name = model_name
        self.base_path = base_path
        self.model_name_capital = model_name.capitalize()
        self.model_name_lower = model_name.lower()

    def write_file(self, filename: str, content: str) -> None:
        target_dir = self.base_path
        full_path = os.path.join(target_dir, filename)

        ensure_package(target_dir)
        if os.path.exists(full_path):
            return

        with open(full_path, "w") as f:
            f.write(content)

    @abstractmethod
    def generate(self, fields: list) -> None:
        pass


from django.db import models


class Book(models.Model):
    title = models.CharField(max_length=100)
    author = models.CharField(max_length=100)
    publication_date = models.DateField()
    num_pages = models.IntegerField()
    is_borrowed = models.BooleanField(default=False)

    def __str__(self):
        return self.title



from django_model_suite.generators.base import BaseGenerator


class URLGenerator(BaseGenerator):
    def generate(self, fields: list) -> None:
        content = f"""from django.urls import path
from .views import (
    {self.model_name_capital}ListView,
    {self.model_name_capital}CreateView,
    {self.model_name_capital}DetailView,
    {self.model_name_capital}UpdateView,
    {self.model_name_capital}DeleteView
)

urlpatterns = [
    path('{self.model_name_lower}s/', {self.model_name_capital}ListView.as_view(), name='{self.model_name_lower}-list'),
    path('{self.model_name_lower}/create/', {self.model_name_capital}CreateView.as_view(), name='{self.model_name_lower}-create'),
    path('{self.model_name_lower}/<int:id>/', {self.model_name_capital}DetailView.as_view(), name='{self.model_name_lower}-detail'),
    path('{self.model_name_lower}/<int:id>/update/', {self.model_name_capital}UpdateView.as_view(), name='{self.model_name_lower}-update'),
    path('{self.model_name_lower}/<int:id>/delete/', {self.model_name_capital}DeleteView.as_view(), name='{self.model_name_lower}-delete'),
]
"""
        self.write_file("urls.py", content)



# Generated by Django 4.2.18 on 2025-02-05 07:58

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Book',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100)),
                ('author', models.CharField(max_length=100)),
                ('publication_date', models.DateField()),
                ('num_pages', models.IntegerField()),
                ('is_borrowed', models.BooleanField(default=False)),
            ],
        ),
    ]



from ..base import BaseGenerator


class ListViewGenerator(BaseGenerator):
    def generate(self, fields: list) -> None:
        content = f'''class {self.model_name_capital}ListView:
    list_display =  ("display_header",)
    list_editable = ()
    list_filter = ()
    date_hierarchy = None
    list_per_page = 50
    list_filter_submit = False
    list_fullwidth = False
    list_horizontal_scrollbar_top = False
    search_fields = ()
    search_help_text = ""
    ordering = ()

    def get_ordering(self, request):
        return ()
'''
        self.write_file('list_view.py', content)